@@ -1,17 +1,19 @@
-# -*- coding: utf-8 -*-
 # yapf
 
 import funcy
 import types
 import numbers
+import copy
 from functools import singledispatch
-from functools import lru_cache
 from pysistence import Expando
-from typing import Any, Mapping, List, Tuple, Iterable, Generator, Callable, Optional, Union
+from typing import Any, Mapping, List, Tuple, Iterable, Generator, Callable, Union
 import inspect
 from . import pipelib
+import itertools
 
-from functools import wraps, update_wrapper, reduce
+from functools import wraps, update_wrapper
+
+import operator as ops
 
 PRIMTYPES = {int, bool, float, str, set, list, tuple, dict}
 LISTLIKE = {set, list, tuple}
@@ -57,6 +59,8 @@ def pubvars(obj: Any) -> Iterable:
     "Returns all public variables except methods"
     if isdict(obj):
         return obj.keys()
+    if funcy.is_seqcoll(obj) or isinstance(obj, set):
+        return copy.copy(obj)
     else:
         return [
             attr for attr in dir(obj)
@@ -133,10 +137,6 @@ def bind_methods(Base, instance):
     return instance
 
 
-__classmethod__ = classmethod
-__staticmethod__ = staticmethod
-
-
 class Base(object):
     __bind__: List[Tuple[Callable, Any]] = []
 
@@ -158,7 +158,7 @@ class Base(object):
 
         self.method = types.MethodType(bind_method_on_self, self)
 
-    @__staticmethod__
+    @staticmethod
     def __wrapper(fn: Callable, Cls: Any = None, name: str = None):
         "Does __wrapper"
         if name is None:
@@ -178,11 +178,13 @@ class Base(object):
 
         return do_call
 
-    @__classmethod__
+    __classmethod__ = classmethod
+
+    @classmethod
     def classmethod(cls, fn):
         return Base.__wrapper(fn, Cls=cls)
 
-    @__staticmethod__
+    @staticmethod
     def staticmethod(fn: Callable) -> Callable:
         return Base.__wrapper(fn)
 
@@ -228,16 +230,20 @@ class Piping(pipelib.BasePiping):
                  seed: Union[tuple, Any] = (),
                  kind: Callable = map,
                  format: Callable[[Any, None, None], Any] = funcy.identity):
-        self.cursor = ()
-        self.ops = ()
-        self.results = {}
-        self.last_result = ()
-
+        "Initialize a new Piping object."
+        self.reset()
         self.format = format
         self.kind = kind
 
         self.seed = always_tup(seed)
 
+    def reset(self):
+        "Restart this Piping object as if it were new."
+        self.cursor = ()
+        self.ops = ()
+        self.results = {}
+        self.last_result = ()
+
     def sum(self) -> None:
         "Does show"
         print(self.__class__.__name__)
@@ -252,22 +258,16 @@ class Piping(pipelib.BasePiping):
 
     def fncompose(self, stepf: Callable[[Any, None, None], Any],
                   x: Any = None) -> 'Piping':
-        operands = arity(stepf)
-        if operands == 1:
-            operand = ()  # only return value from previous step function
-        elif operands == 2:
-            operand = (x, )
-
         self.queue(stepf, x)
         return self
 
-    def queue(self, stepf: Callable[[Any, Any, None], Any], *x: Any) -> None:
+    def queue(self, stepf: Callable[[Any, None, None], Any],
+              *x: Any) -> 'Piping':
         "Does queue"
-        cursor = self.cursor  # at time of queue
         self.ops = (*self.ops, ((stepf, x)))
         return self
 
-    def run(self, seed, *x: Any) -> None:
+    def run(self, seed, *x: Any) -> Any:
         "Does do"
         self.cursor = seed
         for op, operands in self.ops:
@@ -357,18 +357,128 @@ class ComposePiping(Piping):
         return self.queue(stepf)
 
 
-class callbytype(dict):
-    """Crue type match and extractor:
+class CountPiping(Piping):
+    def __add__(self, value) -> Piping:
+        "Add operation"
+        return self.queue(ops.add, value)
+
+    def __sub__(self, value) -> Piping:
+        "Add operation"
+        return self.queue(ops.sub, value)
+
+    def __mul__(self, value) -> Piping:
+        "Add operation"
+        return self.queue(ops.mul, value)
+
+    def __div__(self, value) -> Piping:
+        "Add operation"
+        return self.queue(ops.truediv, value)
+
+
+def PNot(value_or_stepf: Union[Callable, Any]) -> bool:
+    "Unary negate for `LogicPiping`."
+    if callable(value_or_stepf):
+        stepf = value_or_stepf
+    else:
+        stepf = funcy.identity
+    return funcy.complement(stepf)
+
+
+rcurry = funcy.rcurry
+
+
+class LogicPiping(Piping):
+    def __init__(self,
+                 seed: Union[tuple, Any] = (),
+                 kind: Callable = map,
+                 format: Callable[[Any, None, None], Any] = funcy.identity):
+        "Sensible default, override for advanced use."
+        super().__init__(seed, kind, format)
+        self.truthy = []
+        self.conjunctions = 0
+
+    def logically(self, stepf, conjunction):
+        self.counter = itertools.count(1)
+        if conjunction:
+            self.conjunctions += 1
+
+        def gate(x):
+            if stepf(x):
+                self.truthy.append(x)
+            else:
+                self.truthy.append(False)
+            return x
+
+        return self.queue(gate)
+
+    def __call__(self, *params: Any) -> Any:
+        self.truthy = []
+        super().__call__(*params)
+        passed = funcy.compact(self.truthy)
+        if len(passed) == self.conjunctions:
+            return self.format(passed[-1])
+        else:
+            return False
+
+    def __eq__(self, value) -> None:
+        "Does __ge__"
+        return self.logically(rcurry(ops.eq)(value), True)
+
+    def __neg__(self, stepf) -> None:
+        "Does __ge__"
+        return self.logically(funcy.complement(stepf), False)
+
+    def __ne__(self, value) -> None:
+        "Does __ge__"
+        return self.logically(rcurry(ops.ne)(value), True)
+
+    def __ge__(self, value) -> None:
+        "Does __ge__"
+        return self.logically(rcurry(ops.ge)(value), True)
+
+    def __gt__(self, value) -> None:
+        "Does __ge__"
+        return self.logically(rcurry(ops.gt)(value), True)
+
+    def __and__(self, stepf: Callable[[Any, None, None], Any]) -> Piping:
+        return self.logically(stepf, True)
+
+    def __floordiv__(self, stepf: Callable[[Any, None, None], Any]) -> Piping:
+        "Aestetics only."
+        return self.__and__(stepf)
+
+    def __or__(self, stepf: Callable[[Any, None, None], Any]) -> Piping:
+        return self.logically(stepf, False)
+
+
+class Match(dict):
+    """Crue type Match and extractor:
 
     Define a mapping of types. Call for an instance, the parameter
     type should be mapped by `type(obj)` returning a callable that
     will further process the instance.
 
     """
+    def case(*params: Any) -> Callable:
+        """Decorator to add a function. The types of the parameters. The types
+        that will be matched is taken from the signature of the
+        decorated function.
+
+        """
+        self, fn = params
+        disp = tuple(arg.annotation
+                     for arg in inspect.signature(fn).parameters.values())[1:]
+
+        self[disp] = fn
+
+        return fn
+
     def __call__(self, *params: Any,
                  **opts: Any) -> Union[Iterable[Any], Iterable]:
         "Return the value keyed by the type of parameter `obj`"
-        instance = params[0]
-        T = type(instance)
 
-        return self[T](instance, *params[1:], **opts)
+        T = tuple(type(p) for p in params)
+        if len(T) == 1:
+            return self[T[0]](*params)
+        else:
+            return self[T](self, *params)
