@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 # yapf
 
 import sys
 import io
-from typing import Any, Tuple, List, Mapping, Callable
+from typing import Any, Tuple, List, Mapping, Callable, Optional
+import pdb
+import traceback
 
 import distutils.cmd
 
@@ -28,48 +29,7 @@ class fixture(object):
         return pytest.mark.parametrize(namelist, values)
 
 
-def logcall(fn: Callable, realname: str = None) -> Callable:
-    "A simple composition that will log parameters an results."
-
-    @functools.wraps(fn)
-    def _(*args: Any, **kwargs: Mapping[Any, Any]) -> Any:
-        "Function logger, will log if enclosing functions is turned on"
-        on = logcall._enabled
-        try:
-            name = realname or fn.__name__
-            if kwargs:
-                desc = f'{name}({args}, {kwargs}'
-            else:
-                desc = desc = f'{name}({args})'
-            if on:
-                print(desc)
-            ret = fn(*args, **kwargs)
-            if on:
-                print(f'  -> {ret}')
-            return ret
-        except Exception as exc:
-            if on:
-                print(f'  ..raised {exc}')
-            raise
-
-    @contextmanager
-    def on():
-        logcall._enabled = True
-        try:
-            yield
-        except Exception as exc:
-            raise
-        finally:
-            logcall._enabled = False
-
-        logcall._enabled = False
-
-    logcall.on = on
-    logcall._enabled = False
-    return _
-
-
-class ReviewProject(distutils.cmd.Command):
+class ReviewProject(distutils.cmd.Command):  # pragma: nocov
     user_options: List[str] = []
 
     def initialize_options(self: 'ReviewProject') -> None:
@@ -139,3 +99,26 @@ class ReviewProject(distutils.cmd.Command):
             ReviewProject.separator_line()
             print()
             print(f'{issues} issues found.')
+
+
+def hook_uncatched(pm_func: Optional[Callable] = None
+                   ) -> None:  # pragma: nocov
+    """Installs an exception hook that triggers a debugger post-mortem on
+    unhandled exceptions.
+
+    """
+
+    if pm_func is None:
+        pm_func = pdb.post_mortem
+
+    def unhandled(type_: Any, value: Any, tb: Any) -> None:
+        """Catches unhandled exceptions, print stack trace and executes
+        debugger post-mortem entry point.
+
+        """
+        traceback.print_exception(type_, value, tb)
+        print()  # lf
+        pm_func(tb)
+
+    sys.excepthook = unhandled
+    return unhandled
